
use rocket::{*, fairing::AdHoc};
use rocket_dyn_templates::Template;
use ::serde::Deserialize;
use tools::tera::{static_filter, translate_filter, calculate_age, insert_space_every, lang_entity, gravatar_link, format_date};

pub mod routes;
pub mod services;
pub mod tools;

#[derive(Deserialize)]
#[serde(crate = "rocket::serde")]
pub struct PresentationConfig {
    static_route: String,
}

#[derive(Deserialize)]
#[serde(crate = "rocket::serde")]
pub struct CVBackendConfig {
    cv_backend_path: String,
}


pub fn rocket_builder() -> Rocket<Build> {  
    let rocket =  rocket::build();
    //let figment = rocket.figment();
      // extract the entire config any `Deserialize` value
    //let config: PresentationConfig = figment.extract().expect("config");
    
    rocket.attach(
        Template::try_custom(|engines| {
            engines.tera.register_filter("static", static_filter);
            engines.tera.register_filter("translate", translate_filter);
            engines.tera.register_filter("calculate_age", calculate_age);
            engines.tera.register_filter("insert_space_every", insert_space_every);
            engines.tera.register_filter("gravatar_link", gravatar_link);
            // filters specific to API
            engines.tera.register_filter("lang_entity", lang_entity);
            engines.tera.register_filter("format_date", format_date);
            Ok(())
        })
    ).attach(
        AdHoc::config::<PresentationConfig>()
    ).attach(
        AdHoc::config::<CVBackendConfig>()
    ).attach(
        tools::rocket::RequestTimer
    ).mount("/", routes![
        routes::root::index,
        routes::root::index_without_lang
    ])
}