use std::{collections::HashMap, str::FromStr};

use chrono::{Utc, Datelike, NaiveDate};
use log::error;
use rocket_dyn_templates::tera::{Value, Error};
use ovlach_data::cv::chrono::from_string;
use sha256::digest;
use fluent_resmgr::resource_manager::ResourceManager;
use unic_langid::LanguageIdentifier;

// TODO: tenhle modul je trochu prasacky..

pub fn static_filter(
    value: &Value,
    args: &HashMap<String, rocket_dyn_templates::tera::Value>
) -> Result<Value, Error> {
    let host = args.get("static_host");
    return Ok(rocket_dyn_templates::tera::Value::String(format!("{}/{}", host.unwrap().as_str().unwrap(), value.as_str().unwrap()))); // TODO: fix-me here!
}

pub fn translate_filter(
    value: &Value,
    args: &HashMap<String, rocket_dyn_templates::tera::Value>
) -> Result<Value, Error> {
    // TODO: prepis me!
    let mgr = ResourceManager::new("./resources/{locale}/{res_id}".into());
    let lang = args["lang"].as_str().unwrap();
    let locales = vec![LanguageIdentifier::from_str(lang).unwrap()];
    let val = value.as_str().unwrap().to_string();
    
    let bundle = mgr.get_bundle(locales, vec!["ovlach_frontend".to_string()]);

    let mut errors = vec![];

    match bundle.get_message(&val) {
        Some(msg) => {
            match msg.value() {
                Some(pattern) => {
                    let output = bundle.format_pattern(pattern, None, &mut errors);
                    return Ok(rocket_dyn_templates::tera::Value::String(output.clone().to_string()))
                },
                None => {
                    error!("Can't translate message: {}", val);
                    return Ok(rocket_dyn_templates::tera::Value::String(format!("{}", val)));
                },
            }
        },
        None =>  {
            error!("Can't translate message: {}", val);
            return Ok(rocket_dyn_templates::tera::Value::String(format!("{}", val)));
        },
    };
}

pub fn lang_entity(
    value: &Value,
    _: &HashMap<String, rocket_dyn_templates::tera::Value>
) -> Result<Value, Error> {
    // TODO: rewrite me!
    return Ok(rocket_dyn_templates::tera::Value::String(format!("{}", value["en"].as_str().unwrap()))); // TODO: fix-me here!
}

pub fn gravatar_link(
    value: &Value,
    _: &HashMap<String, rocket_dyn_templates::tera::Value>
) -> Result<Value, Error> {
    let val = digest(value.as_str().unwrap());
    return Ok(rocket_dyn_templates::tera::Value::String(format!("https://gravatar.com/avatar/{}", val))); // TODO: fix-me here!
}

pub fn format_date(
    value: &Value,
    _: &HashMap<String, rocket_dyn_templates::tera::Value>
) -> Result<Value, Error> {
    
    if value.is_null() {
        return Ok(rocket_dyn_templates::tera::Value::String("".to_string()))
    }
    let val = value.as_str().unwrap();
    
    if val == "" {
        return Ok(rocket_dyn_templates::tera::Value::String("".to_string()))
    }

    let value =  from_string(value.as_str().unwrap().into());
    match value {
        Ok(date) => Ok(rocket_dyn_templates::tera::Value::String(date.format("%Y").to_string())),
        Err(e) => {
                error!("Can't parse date: {}", e);
                return Ok(rocket_dyn_templates::tera::Value::String("".to_string())) // TOdo: error
        }
    }
}

pub fn insert_space_every(
        value: &Value,
        args: &HashMap<String, rocket_dyn_templates::tera::Value>
) -> Result<Value, Error> {
    let input = value.as_u64().unwrap().to_string();
    let times = args.get("times").unwrap().as_u64().unwrap();
    let mut result = String::new();
    
    for (index, ch) in input.chars().enumerate() {
        if index > 0 && index as u64 % times == 0 {
            result.push(' ');
        }
        result.push(ch);
    }

    Ok(rocket_dyn_templates::tera::Value::String(result))
}

pub fn calculate_age(
    value: &Value,
    _: &HashMap<String, rocket_dyn_templates::tera::Value>
) -> Result<Value, Error> {
    //let s = value.to_string().trim_matches('"'); // TODO: unwrap here!
    let value =  from_string(value.as_str().unwrap().into());
    match value {
        Ok(value) => {
            let current_date = Utc::now().naive_utc().date();
            let mut age = current_date.year() - value.year();
        
            // Adjust age if the birthday hasn't occurred yet this year
            age = if current_date.month() < value.month() || 
               (current_date.month() == value.month() && current_date.day() < value.day()) {
                age - 1
            } else {
                age
            };

            Ok(rocket_dyn_templates::tera::Value::String(age.to_string()))
        }
        Err(e) => panic!("Can't parse date: {}", e)
    }
}
